@@ -18,6 +18,10 @@
 #include <sys/un.h>
 #include <signal.h>
 #include <sys/ioctl.h>
+#include <sys/resource.h> // limit
+#include <stdint.h>
+#include <sys/prctl.h>
+
 
 #include <oblibs/log.h>
 #include <oblibs/string.h>
@@ -38,6 +42,7 @@
 #include <66/service.h>
 #include <66/constants.h>
 #include <66/utils.h>
+#include <66/caps.h>
 
 #include <s6/fdholder.h>
 
@@ -87,7 +92,8 @@ static void execute_setup_destination(resolve_service_t *res, const char *dest)
     uid_t uid ;
     gid_t gid ;
 
-    setup_uidgid(&uid, &gid, res, res->islog ? res->logger.execute.run.runas : 0) ;
+    // root by default as !res->logger.execute.run.runas if not set
+    setup_uidgid(&uid, &gid, res, res->logger.execute.run.runas) ;
 
     log_trace("check logger destination directory: ", dest) ;
     if (!dir_create_parent(dest, 0755))
@@ -178,7 +184,7 @@ static void io_open_file(resolve_service_t *res,  int fd, char const *destinatio
         log_dieusys(LOG_EXIT_SYS, "remove close-on-exec from fd") ;
 }
 
-static void io_open_destination(int fd, const char *destination, int flags, mode_t mode, uint8_t cloexec)
+static void io_open_destination(int fd, const char *destination, int flags, uint8_t cloexec)
 {
     log_flow() ;
 
@@ -234,19 +240,19 @@ static void io_read_file(stack *stk, const char *file, size_t len)
     stk->s[n] = 0 ;
 }
 
-static void io_open_terminal(resolve_service_t *res, int fd, const char *destination, int flags)
+static void io_open_terminal(int fd, const char *destination, int flags)
 {
     log_flow() ;
 
     int e = errno ;
-    io_open_destination(fd, destination, flags, 0, 0) ;
+    io_open_destination(fd, destination, flags, 0) ;
     errno = 0 ;
     if (!isatty(fd) && errno != EBADF)
         log_dieusys(LOG_EXIT_ZERO, "associate fd to: ", destination) ;
     errno = e ;
 }
 
-static void io_open_terminal_ncontrol(resolve_service_t *res, int fd, const char *destination, int flags)
+static void io_open_terminal_ncontrol(int fd, const char *destination, int flags)
 {
     log_flow() ;
 
@@ -258,7 +264,7 @@ static void io_open_terminal_ncontrol(resolve_service_t *res, int fd, const char
     sigemptyset(&snew.sa_mask) ;
     snew.sa_flags = 0 ;
 
-    io_open_terminal(res, fd, destination, flags) ;
+    io_open_terminal(fd, destination, flags) ;
 
     if (sigaction(SIGTTOU, &snew, &sold) < 0)
         log_dieusys(LOG_EXIT_SYS, "ignore SIGTTOU") ;
@@ -281,7 +287,7 @@ static void io_open_terminal_ncontrol(resolve_service_t *res, int fd, const char
     errno = e ;
 }
 
-static void io_open_active_console(int fd, resolve_service_t *res)
+static void io_open_active_console(int fd)
 {
     log_flow() ;
 
@@ -299,7 +305,7 @@ static void io_open_active_console(int fd, resolve_service_t *res)
 
     stk.s[stk.len - 1] = 0 ; // remove the last '\n'
 
-    io_open_terminal(res, fd, stk.s, O_WRONLY | O_NOCTTY) ;
+    io_open_terminal(fd, stk.s, O_WRONLY | O_NOCTTY) ;
     errno = e ;
 }
 
@@ -353,11 +359,11 @@ static void io_setup_stdin(resolve_service_t *res)
     switch(res->io.fdin.type) {
 
         case E_PARSER_IO_TYPE_TTY:
-            io_open_terminal_ncontrol(res, 0, res->sa.s + res->io.fdin.destination, O_RDWR | O_NOCTTY) ;
+            io_open_terminal_ncontrol(0, res->sa.s + res->io.fdin.destination, O_RDWR | O_NOCTTY) ;
             break ;
 
         case E_PARSER_IO_TYPE_NULL:
-            io_open_destination(0, res->sa.s +  res->io.fdin.destination, O_RDONLY | O_NOCTTY, 0, 0) ;
+            io_open_destination(0, res->sa.s +  res->io.fdin.destination, O_RDONLY | O_NOCTTY, 0) ;
             break ;
 
         case E_PARSER_IO_TYPE_CLOSE:
@@ -390,7 +396,7 @@ static void io_setup_stdout(resolve_service_t *res)
     switch(res->io.fdout.type) {
 
         case E_PARSER_IO_TYPE_CONSOLE:
-            io_open_active_console(1, res) ;
+            io_open_active_console(1) ;
             break ;
 
         case E_PARSER_IO_TYPE_S6LOG:
@@ -401,20 +407,20 @@ static void io_setup_stdout(resolve_service_t *res)
 
             } else if (res->type == E_PARSER_TYPE_ONESHOT) {
 
-                _alloc_stk_(stk, strlen(res->sa.s + res->io.fdout.destination) + SS_TREE_CURRENT_LEN + 2) ;
-                auto_strings(stk.s, res->sa.s + res->io.fdout.destination, "/", SS_TREE_CURRENT) ;
-                stk.s[strlen(res->sa.s + res->io.fdout.destination) + 1 + SS_TREE_CURRENT_LEN] = 0 ;
+                _alloc_stk_(stk, strlen(res->sa.s + res->io.fdout.destination) + SS_CURRENT_LEN + 2) ;
+                auto_strings(stk.s, res->sa.s + res->io.fdout.destination, "/", SS_CURRENT) ;
+                stk.s[strlen(res->sa.s + res->io.fdout.destination) + 1 + SS_CURRENT_LEN] = 0 ;
 
                 io_open_file(res, 1, stk.s) ;
             }
             break ;
 
         case E_PARSER_IO_TYPE_TTY:
-            io_open_terminal(res, 1, res->sa.s + res->io.fdout.destination, O_WRONLY | O_NOCTTY) ;
+            io_open_terminal(1, res->sa.s + res->io.fdout.destination, O_WRONLY | O_NOCTTY) ;
             break ;
 
         case E_PARSER_IO_TYPE_NULL:
-            io_open_destination(1, res->sa.s + res->io.fdout.destination, O_WRONLY, 0, 0) ;
+            io_open_destination(1, res->sa.s + res->io.fdout.destination, O_WRONLY, 0) ;
             break ;
 
         case E_PARSER_IO_TYPE_FILE:
@@ -450,15 +456,15 @@ static void io_setup_stderr(resolve_service_t *res)
     switch(res->io.fderr.type) {
 
         case E_PARSER_IO_TYPE_CONSOLE:
-            io_open_active_console(2, res) ;
+            io_open_active_console(2) ;
             break ;
 
         case E_PARSER_IO_TYPE_TTY:
-            io_open_terminal(res, 2, res->sa.s + res->io.fderr.destination, O_WRONLY | O_NOCTTY) ;
+            io_open_terminal(2, res->sa.s + res->io.fderr.destination, O_WRONLY | O_NOCTTY) ;
             break ;
 
         case E_PARSER_IO_TYPE_NULL:
-            io_open_destination(2, res->sa.s + res->io.fdout.destination, O_WRONLY, 0, 0) ;
+            io_open_destination(2, res->sa.s + res->io.fdout.destination, O_WRONLY, 0) ;
             break ;
 
         case E_PARSER_IO_TYPE_FILE:
@@ -509,12 +515,12 @@ static void execute_environment(char const **nenvp, char const *const *env, stra
             size_t pos = 0 ;
 
             if (!stack_string_clean(&stk, res->sa.s + res->environ.importfile))
-                log_dieusys(LOG_EXIT_ZERO, "clean string") ;
+                log_dieusys(LOG_EXIT_SYS, "clean string") ;
 
             FOREACH_STK(&stk, pos) {
 
                 if (!environ_merge_file(eram, stk.s + pos))
-                    log_dieusys(LOG_EXIT_ZERO, "merge environment file: ", stk.s + pos) ;
+                    log_dieusys(LOG_EXIT_SYS, "merge environment file: ", stk.s + pos) ;
             }
         }
 
@@ -633,6 +639,137 @@ static void execute_uidgid(resolve_service_t *res)
     }
 }
 
+static void limit_setup(resolve_service_t *res, int resource, uint64_t rval)
+{
+    log_flow() ;
+
+    if (!rval)
+        return ;
+
+    struct rlimit r ;
+    uint64_t n = rval ;
+
+    if (getrlimit(resource, &r) < 0)
+        log_dieusys(LOG_EXIT_SYS, "get limit") ;
+
+    // n == (uint64_t)(RLIM_INFINITY) is implied
+    if (!r.rlim_max && !res->owner)
+        r.rlim_max = n ;
+
+    if (n > r.rlim_max)
+        if (n == (uint64_t)(RLIM_INFINITY) && !res->owner)
+            r.rlim_max = n ;
+        else n = r.rlim_max ;
+
+    r.rlim_cur = n ;
+
+    if (setrlimit(resource, &r) < 0)
+        log_dieusys(LOG_EXIT_SYS, "set limit") ;
+}
+
+static void execute_limit(resolve_service_t *res)
+{
+    log_flow() ;
+
+#ifdef RLIMIT_AS
+    limit_setup(res, RLIMIT_AS, res->limit.limitas ? res->limit.limitas : 0) ;
+#endif
+#ifdef RLIMIT_CORE
+    limit_setup(res, RLIMIT_CORE, res->limit.limitcore ? res->limit.limitcore : 0) ;
+#endif
+#ifdef RLIMIT_CPU
+    limit_setup(res, RLIMIT_CPU, res->limit.limitcpu ? res->limit.limitcpu : 0) ;
+#endif
+#ifdef RLIMIT_DATA
+    limit_setup(res, RLIMIT_DATA, res->limit.limitdata ? res->limit.limitdata : 0) ;
+#endif
+#ifdef RLIMIT_FSIZE
+    limit_setup(res, RLIMIT_FSIZE, res->limit.limitfsize ? res->limit.limitfsize : 0) ;
+#endif
+#ifdef RLIMIT_LOCKS
+    limit_setup(res, RLIMIT_LOCKS, res->limit.limitlocks ? res->limit.limitlocks : 0) ;
+#endif
+#ifdef RLIMIT_MEMLOCK
+    limit_setup(res, RLIMIT_MEMLOCK, res->limit.limitmemlock ? res->limit.limitmemlock : 0) ;
+#endif
+#ifdef RLIMIT_MSGQUEUE
+    limit_setup(res, RLIMIT_MSGQUEUE, res->limit.limitmsgqueue ? res->limit.limitmsgqueue : 0) ;
+#endif
+#ifdef RLIMIT_NICE
+    limit_setup(res, RLIMIT_NICE, res->limit.limitnice ? res->limit.limitnice : 0) ;
+#endif
+#ifdef RLIMIT_NOFILE
+    limit_setup(res, RLIMIT_NOFILE, res->limit.limitnofile ? res->limit.limitnofile : 0) ;
+#endif
+#ifdef RLIMIT_NPROC
+    limit_setup(res, RLIMIT_NPROC, res->limit.limitnproc ? res->limit.limitnproc : 0) ;
+#endif
+#ifdef RLIMIT_RTPRIO
+    limit_setup(res, RLIMIT_RTPRIO, res->limit.limitrtprio ? res->limit.limitrtprio : 0) ;
+#endif
+#ifdef RLIMIT_RTTIME
+    limit_setup(res, RLIMIT_RTTIME, res->limit.limitrttime ? res->limit.limitrttime : 0) ;
+#endif
+#ifdef RLIMIT_SIGPENDING
+    limit_setup(res, RLIMIT_SIGPENDING, res->limit.limitsigpending ? res->limit.limitsigpending : 0) ;
+#endif
+#ifdef RLIMIT_STACK
+    limit_setup(res, RLIMIT_STACK, res->limit.limitstack ? res->limit.limitstack : 0) ;
+#endif
+
+}
+
+static void execute_privileges(resolve_service_t *res)
+{
+    log_flow() ;
+
+    if (res->execute.blockprivileges)
+        if (prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0) < 0)
+            log_dieusys(LOG_EXIT_SYS, "set NO_NEW_PRIVILEGES") ;
+}
+
+static void execute_umask(resolve_service_t *res)
+{
+    log_flow() ;
+
+    if (res->execute.want_umask)
+        umask((mode_t)res->execute.umask) ;
+}
+
+static void execute_nice(resolve_service_t *res)
+{
+    log_flow();
+
+    if (res->execute.want_nice) {
+
+        int64_t p = 20 - (int64_t)(res->execute.nice) ;
+        /** were are root and service is badly define. Be smart
+         * and respect the limit defined by the service itself. */
+        if (!res->owner && (res->execute.nice > res->limit.limitnice))
+            p = 20 - res->limit.limitnice ;
+
+        errno = 0 ; // see: https://pubs.opengroup.org/onlinepubs/9699919799/
+        if (setpriority(PRIO_PROCESS, 0, (int)p) < 0) {
+            if (errno == EPERM)
+                log_warnusys("setting nice value requires CAP_SYS_NICE or root") ;
+
+            log_dieusys(LOG_EXIT_SYS, "set nice value");
+        }
+    }
+}
+
+static void execute_chdir(resolve_service_t *res)
+{
+    log_flow() ;
+
+    if (res->execute.chdir) {
+
+        if (chdir(res->sa.s + res->execute.chdir) < 0)
+            log_dieusys(LOG_EXIT_ZERO, "chdir") ;
+
+    }
+}
+
 int main(int argc, char const *const *argv, char const *const *envp)
 {
     log_flow() ;
@@ -717,18 +854,30 @@ int main(int argc, char const *const *argv, char const *const *envp)
             log_dieusys(LOG_EXIT_SYS, "find script: ", brun, " -- please make a bug report") ;
     }
 
-    execute_environment(nenvp, envp, &eram, &info, &res) ;
-
-    execute_script(brunuser, &res, &info) ;
-
     execute_io(&res) ;
 
-    execute_uidgid(&res) ;
-
     /** We can now send message to a eventd handler socket.
      * For now, just send a simple message */
     log_info(action == EXECUTE_START ? "Starting" : "Stopping", " service: ", service) ;
 
+    execute_environment(nenvp, envp, &eram, &info, &res) ;
+
+    execute_script(brunuser, &res, &info) ;
+
+    execute_limit(&res) ;
+
+    execute_nice(&res) ;
+
+    execute_privileges(&res) ;
+
+    execute_caps(&res) ;
+
+    execute_uidgid(&res) ;
+
+    execute_umask(&res) ;
+
+    execute_chdir(&res) ;
+
     xmexec_em(newargv, nenvp, info.modifs.s, info.modifs.len) ;
 
     return 0 ;
