@@ -36,7 +36,7 @@
 #include <66/config.h>
 #include <66/state.h>
 
-#define MAXOPTS 81
+#define MAXOPTS 108
 
 static wchar_t const field_suffix[] = L" :" ;
 static char fields[INFO_NKEY][INFO_FIELD_MAXLEN] = {{ 0 }} ;
@@ -62,6 +62,20 @@ static void info_display_string(char const *field, char const *str, uint32_t ele
 
 }
 
+static void info_display_int64(char const *field, uint64_t element)
+{
+    info_display_field_name(field) ;
+
+    char ui[UINT64_FMT] ;
+    ui[uint64_fmt(ui, element)] = 0 ;
+
+    if (!buffer_puts(buffer_1, ui))
+        log_dieusys(LOG_EXIT_SYS, "write to stdout") ;
+
+    if (buffer_putsflush(buffer_1, "\n") == -1)
+        log_dieusys(LOG_EXIT_SYS, "write to stdout") ;
+}
+
 static void info_display_int(char const *field, uint32_t element)
 {
     info_display_field_name(field) ;
@@ -105,11 +119,13 @@ static void info_display_service_field(resolve_service_t *res)
     info_display_string(fields[m++], res->sa.s, res->dependencies.optsdeps, 1) ;
     info_display_string(fields[m++], res->sa.s, res->dependencies.contents, 1) ;
     info_display_string(fields[m++], res->sa.s, res->dependencies.provide, 1) ;
+    info_display_string(fields[m++], res->sa.s, res->dependencies.conflict, 1) ;
     info_display_int(fields[m++], res->dependencies.ndepends) ;
     info_display_int(fields[m++], res->dependencies.nrequiredby) ;
     info_display_int(fields[m++], res->dependencies.noptsdeps) ;
     info_display_int(fields[m++], res->dependencies.ncontents) ;
     info_display_int(fields[m++], res->dependencies.nprovide) ;
+    info_display_int(fields[m++], res->dependencies.nconflict) ;
 
     info_display_string(fields[m++], res->sa.s, res->execute.run.run, 1) ;
     info_display_string(fields[m++], res->sa.s, res->execute.run.run_user, 1) ;
@@ -123,6 +139,31 @@ static void info_display_service_field(resolve_service_t *res)
     info_display_int(fields[m++], res->execute.timeout.stop) ;
     info_display_int(fields[m++], res->execute.down) ;
     info_display_int(fields[m++], res->execute.downsignal) ;
+    info_display_int(fields[m++], res->execute.blockprivileges) ;
+    info_display_int(fields[m++], res->execute.umask) ;
+    info_display_int(fields[m++], res->execute.want_umask) ;
+    info_display_int(fields[m++], res->execute.nice) ;
+    info_display_int(fields[m++], res->execute.want_nice) ;
+    info_display_string(fields[m++], res->sa.s, res->execute.chdir, 1) ;
+    info_display_int64(fields[m++],res->limit.limitas) ;
+    info_display_int64(fields[m++],res->limit.limitcore) ;
+    info_display_int64(fields[m++],res->limit.limitcpu) ;
+    info_display_int64(fields[m++],res->limit.limitdata) ;
+    info_display_int64(fields[m++],res->limit.limitfsize) ;
+    info_display_int64(fields[m++],res->limit.limitlocks) ;
+    info_display_int64(fields[m++],res->limit.limitmemlock) ;
+    info_display_int64(fields[m++],res->limit.limitmsgqueue) ;
+    info_display_int64(fields[m++],res->limit.limitnice) ;
+    info_display_int64(fields[m++],res->limit.limitnofile) ;
+    info_display_int64(fields[m++],res->limit.limitnproc) ;
+    info_display_int64(fields[m++],res->limit.limitrtprio) ;
+    info_display_int64(fields[m++],res->limit.limitrttime) ;
+    info_display_int64(fields[m++],res->limit.limitsigpending) ;
+    info_display_int64(fields[m++],res->limit.limitstack) ;
+    info_display_string(fields[m++], res->sa.s, res->execute.capsbound, 1) ;
+    info_display_string(fields[m++], res->sa.s, res->execute.capsambient, 1) ;
+    info_display_int(fields[m++], res->execute.ncapsbound) ;
+    info_display_int(fields[m++], res->execute.ncapsambient) ;
 
     info_display_string(fields[m++], res->sa.s, res->live.livedir, 1) ;
     info_display_string(fields[m++], res->sa.s, res->live.status, 1) ;
@@ -167,8 +208,8 @@ static void info_display_service_field(resolve_service_t *res)
     info_display_string(fields[m++], res->sa.s, res->io.fdout.destination, 1) ;
     info_display_int(fields[m++], res->io.fderr.type) ;
     info_display_string(fields[m++], res->sa.s, res->io.fderr.destination, 1) ;
-    info_display_string(fields[m], res->sa.s, res->rversion, 1) ;
 
+    info_display_string(fields[m], res->sa.s, res->rversion, 1) ;
 }
 
 int ssexec_resolve(int argc, char const *const *argv, ssexec_t *info)
@@ -207,11 +248,13 @@ int ssexec_resolve(int argc, char const *const *argv, ssexec_t *info)
         "optsdeps",
         "contents",
         "provide",
+        "conflict",
         "ndepends",
         "nrequiredby",
         "noptsdeps",
         "ncontents",
         "nprovide",
+        "nconflict",
 
         "run",
         "run_user",
@@ -225,6 +268,31 @@ int ssexec_resolve(int argc, char const *const *argv, ssexec_t *info)
         "timeoutstop",
         "down",
         "downsignal",
+        "blockprivileges",
+        "umask",
+        "want_umask",
+        "nice",
+        "want_nice",
+        "chdir",
+        "limitas",
+        "limitcore",
+        "limitcpu",
+        "limitdata",
+        "limitfsize",
+        "limitlocks",
+        "limitmemlock",
+        "limitmsgqueue",
+        "limitnice",
+        "limitnofile",
+        "limitnproc",
+        "limitrtprio",
+        "limitrttime",
+        "limitsigpending",
+        "limitstack",
+        "capsbound",
+        "capsambient",
+        "ncapsbound",
+        "ncapsambient",
 
         "livedir",
         "status",
@@ -269,7 +337,8 @@ int ssexec_resolve(int argc, char const *const *argv, ssexec_t *info)
         "stdoutdest",
         "stderrtype",
         "stderrdest",
-        "rversion",
+
+        "rversion"
     } ;
 
     {
